@@ -1,5 +1,4 @@
 <?php
-
 /**
  * Zend Framework
  *
@@ -16,9 +15,9 @@
  * @category   Zend
  * @package    Zend_Auth
  * @subpackage Zend_Auth_Adapter_Http
- * @copyright  Copyright (c) 2005-2007 Zend Technologies USA Inc. (http://www.zend.com)
+ * @copyright  Copyright (c) 2005-2008 Zend Technologies USA Inc. (http://www.zend.com)
  * @license    http://framework.zend.com/license/new-bsd     New BSD License
- * @version    $Id: Http.php 5765 2007-07-18 22:00:00Z thomas $
+ * @version    $Id: Http.php 9404 2008-05-07 23:42:02Z doctorrock83 $
  */
 
 
@@ -36,7 +35,7 @@ require_once 'Zend/Auth/Adapter/Interface.php';
  * @category   Zend
  * @package    Zend_Auth
  * @subpackage Zend_Auth_Adapter_Http
- * @copyright  Copyright (c) 2005-2007 Zend Technologies USA Inc. (http://www.zend.com)
+ * @copyright  Copyright (c) 2005-2008 Zend Technologies USA Inc. (http://www.zend.com)
  * @license    http://framework.zend.com/license/new-bsd     New BSD License
  * @todo       Support auth-int
  * @todo       Track nonces, nonce-count, opaque for replay protection and stale support
@@ -168,6 +167,14 @@ class Zend_Auth_Adapter_Http implements Zend_Auth_Adapter_Interface
      */
     public function __construct(array $config)
     {
+        if (!extension_loaded('hash')) {
+            /**
+             * @see Zend_Auth_Adapter_Exception
+             */
+            require_once 'Zend/Auth/Adapter/Exception.php';
+            throw new Zend_Auth_Adapter_Exception(__CLASS__  . ' requires the \'hash\' extension');
+        }
+
         $this->_request  = null;
         $this->_response = null;
         $this->_ieNoOpaque = false;
@@ -351,8 +358,8 @@ class Zend_Auth_Adapter_Http implements Zend_Auth_Adapter_Interface
     /**
      * Authenticate
      *
-     * @return Zend_Auth_Result
      * @throws Zend_Auth_Adapter_Exception
+     * @return Zend_Auth_Result
      */
     public function authenticate()
     {
@@ -380,30 +387,36 @@ class Zend_Auth_Adapter_Http implements Zend_Auth_Adapter_Interface
         list($clientScheme) = explode(' ', $authHeader);
         $clientScheme = strtolower($clientScheme);
 
+        // The server can issue multiple challenges, but the client should
+        // answer with only the selected auth scheme.
         if (!in_array($clientScheme, $this->_supportedSchemes)) {
             $this->_response->setHttpResponseCode(400);
             return new Zend_Auth_Result(
                 Zend_Auth_Result::FAILURE_UNCATEGORIZED,
                 array(),
-                array('Client requested an unsupported authentication scheme')
+                array('Client requested an incorrect or unsupported authentication scheme')
             );
         }
 
-        // The server can issue multiple challenges, but the client should
-        // answer with only one selected auth scheme.
+        // client sent a scheme that is not the one required
+        if (!in_array($clientScheme, $this->_acceptSchemes)) {
+            // challenge again the client
+            return $this->_challengeClient();
+        }
+        
         switch ($clientScheme) {
             case 'basic':
                 $result = $this->_basicAuth($authHeader);
                 break;
             case 'digest':
                 $result = $this->_digestAuth($authHeader);
-                break;
+            break;
             default:
                 /**
-                 * @see Zend_Auth_Adapter_Exception
-                 */
-                require_once 'Zend/Auth/Adapter/Exception.php';
-                throw new Zend_Auth_Adapter_Exception('Unsupported authentication scheme');
+                * @see Zend_Auth_Adapter_Exception
+                */
+ 	            require_once 'Zend/Auth/Adapter/Exception.php';
+ 	            throw new Zend_Auth_Adapter_Exception('Unsupported authentication scheme');
         }
 
         return $result;
