@@ -14,7 +14,7 @@
  *
  * @category   Zend
  * @package    Zend_Controller
- * @copyright  Copyright (c) 2005-2007 Zend Technologies USA Inc. (http://www.zend.com)
+ * @copyright  Copyright (c) 2005-2008 Zend Technologies USA Inc. (http://www.zend.com)
  * @license    http://framework.zend.com/license/new-bsd     New BSD License
  */
 
@@ -38,6 +38,24 @@ require_once 'Zend/Uri.php';
  */
 class Zend_Controller_Request_Http extends Zend_Controller_Request_Abstract
 {
+    /**
+     * Scheme for http
+     *
+     */
+    const SCHEME_HTTP  = 'http';
+    
+    /**
+     * Scheme for https
+     *
+     */
+    const SCHEME_HTTPS = 'https';
+
+    /**
+     * Allowed parameter sources
+     * @var array
+     */
+    protected $_paramSources = array('_GET', '_POST');
+
     /**
      * REQUEST_URI
      * @var string;
@@ -217,6 +235,29 @@ class Zend_Controller_Request_Http extends Zend_Controller_Request_Abstract
         return $this->__isset($key);
     }
 
+    /**
+     * Set GET values
+     * 
+     * @param  string|array $spec 
+     * @param  null|mixed $value 
+     * @return Zend_Controller_Request_Http
+     */
+    public function setQuery($spec, $value = null)
+    {
+        if ((null === $value) && !is_array($spec)) {
+            require_once 'Zend/Controller/Exception.php';
+            throw new Zend_Controller_Exception('Invalid value passed to setQuery(); must be either array of values or key/value pair');
+        }
+        if ((null === $value) && is_array($spec)) {
+            foreach ($spec as $key => $value) {
+                $this->setQuery($key, $value);
+            }
+            return $this;
+        }
+        $_GET[(string) $spec] = $value;
+        return $this;
+    }
+
     /**
      * Retrieve a member of the $_GET superglobal
      *
@@ -236,6 +277,29 @@ class Zend_Controller_Request_Http extends Zend_Controller_Request_Abstract
         return (isset($_GET[$key])) ? $_GET[$key] : $default;
     }
 
+    /**
+     * Set POST values
+     * 
+     * @param  string|array $spec 
+     * @param  null|mixed $value 
+     * @return Zend_Controller_Request_Http
+     */
+    public function setPost($spec, $value = null)
+    {
+        if ((null === $value) && !is_array($spec)) {
+            require_once 'Zend/Controller/Exception.php';
+            throw new Zend_Controller_Exception('Invalid value passed to setPost(); must be either array of values or key/value pair');
+        }
+        if ((null === $value) && is_array($spec)) {
+            foreach ($spec as $key => $value) {
+                $this->setPost($key, $value);
+            }
+            return $this;
+        }
+        $_POST[(string) $spec] = $value;
+        return $this;
+    }
+
     /**
      * Retrieve a member of the $_POST superglobal
      *
@@ -277,7 +341,7 @@ class Zend_Controller_Request_Http extends Zend_Controller_Request_Abstract
     /**
      * Retrieve a member of the $_SERVER superglobal
      *
-     * If no $key is passed, returns the entire $_COOKIE array.
+     * If no $key is passed, returns the entire $_SERVER array.
      *
      * @param string $key
      * @param mixed $default Default value to use if key not found
@@ -295,7 +359,7 @@ class Zend_Controller_Request_Http extends Zend_Controller_Request_Abstract
     /**
      * Retrieve a member of the $_ENV superglobal
      *
-     * If no $key is passed, returns the entire $_COOKIE array.
+     * If no $key is passed, returns the entire $_ENV array.
      *
      * @param string $key
      * @param mixed $default Default value to use if key not found
@@ -338,12 +402,11 @@ class Zend_Controller_Request_Http extends Zend_Controller_Request_Abstract
             return $this;
         } else {
             // Set GET items, if available
-            $_GET = array();
             if (false !== ($pos = strpos($requestUri, '?'))) {
                 // Get key => value pairs and set $_GET
                 $query = substr($requestUri, $pos + 1);
                 parse_str($query, $vars);
-                $_GET = $vars;
+                $this->setQuery($vars);
             }
         }
 
@@ -560,6 +623,30 @@ class Zend_Controller_Request_Http extends Zend_Controller_Request_Abstract
         return $this->_pathInfo;
     }
 
+    /**
+     * Set allowed parameter sources
+     *
+     * Can be empty array, or contain one or more of '_GET' or '_POST'.
+     * 
+     * @param  array $paramSoures 
+     * @return Zend_Controller_Request_Http
+     */
+    public function setParamSources(array $paramSources = array())
+    {
+        $this->_paramSources = $paramSources;
+        return $this;
+    }
+
+    /**
+     * Get list of allowed parameter sources
+     * 
+     * @return array
+     */
+    public function getParamSources()
+    {
+        return $this->_paramSources;
+    }
+
     /**
      * Set a userland parameter
      *
@@ -572,8 +659,7 @@ class Zend_Controller_Request_Http extends Zend_Controller_Request_Abstract
      */
     public function setParam($key, $value)
     {
-        $keyName = (null !== ($alias = $this->getAlias($key))) ? $alias : $key;
-
+        $key = (null !== ($alias = $this->getAlias($key))) ? $alias : $key;
         parent::setParam($key, $value);
         return $this;
     }
@@ -595,11 +681,12 @@ class Zend_Controller_Request_Http extends Zend_Controller_Request_Abstract
     {
         $keyName = (null !== ($alias = $this->getAlias($key))) ? $alias : $key;
 
+        $paramSources = $this->getParamSources();
         if (isset($this->_params[$keyName])) {
             return $this->_params[$keyName];
-        } elseif ((isset($_GET[$keyName]))) {
+        } elseif (in_array('_GET', $paramSources) && (isset($_GET[$keyName]))) {
             return $_GET[$keyName];
-        } elseif ((isset($_POST[$keyName]))) {
+        } elseif (in_array('_POST', $paramSources) && (isset($_POST[$keyName]))) {
             return $_POST[$keyName];
         }
 
@@ -711,12 +798,130 @@ class Zend_Controller_Request_Http extends Zend_Controller_Request_Abstract
         return false;
     }
 
+    /**
+     * Was the request made by GET?
+     *
+     * @return boolean
+     */
+    public function isGet()
+    {
+        if ('GET' == $this->getMethod()) {
+            return true;
+        }
+
+        return false;
+    }
+
+    /**
+     * Was the request made by PUT?
+     *
+     * @return boolean
+     */
+    public function isPut()
+    {
+        if ('PUT' == $this->getMethod()) {
+            return true;
+        }
+
+        return false;
+    }
+
+    /**
+     * Was the request made by DELETE?
+     *
+     * @return boolean
+     */
+    public function isDelete()
+    {
+        if ('DELETE' == $this->getMethod()) {
+            return true;
+        }
+
+        return false;
+    }
+
+    /**
+     * Was the request made by HEAD?
+     *
+     * @return boolean
+     */
+    public function isHead()
+    {
+        if ('HEAD' == $this->getMethod()) {
+            return true;
+        }
+
+        return false;
+    }
+
+    /**
+     * Was the request made by OPTIONS?
+     *
+     * @return boolean
+     */
+    public function isOptions()
+    {
+        if ('OPTIONS' == $this->getMethod()) {
+            return true;
+        }
+
+        return false;
+    }
+    
+    /**
+     * Is the request a Javascript XMLHttpRequest?
+     *
+     * Should work with Prototype/Script.aculo.us, possibly others.
+     *
+     * @return boolean
+     */
+    public function isXmlHttpRequest()
+    {
+        return ($this->getHeader('X_REQUESTED_WITH') == 'XMLHttpRequest');
+    }
+
+    /**
+     * Is this a Flash request?
+     * 
+     * @return bool
+     */
+    public function isFlashRequest()
+    {
+        return ($this->getHeader('USER_AGENT') == 'Shockwave Flash');
+    }
+    
+    /**
+     * Is https secure request
+     *
+     * @return boolean
+     */
+    public function isSecure()
+    {
+        return ($this->getScheme() === self::SCHEME_HTTPS);
+    }
+    
+    /**
+     * Return the raw body of the request, if present
+     *
+     * @return string|false Raw body, or false if not present
+     */
+    public function getRawBody()
+    {
+        $body = file_get_contents('php://input');
+
+        if (strlen(trim($body)) > 0) {
+            return $body;
+        }
+
+        return false;
+    }
+
     /**
      * Return the value of the given HTTP header. Pass the header name as the
      * plain, HTTP-specified header name. Ex.: Ask for 'Accept' to get the
      * Accept header, 'Accept-Encoding' to get the Accept-Encoding header.
      *
-     * @param string HTTP header name
+     * @param string $header HTTP header name
      * @return string|false HTTP header value, or false if not found
      * @throws Zend_Controller_Request_Exception
      */
@@ -744,16 +949,41 @@ class Zend_Controller_Request_Http extends Zend_Controller_Request_Abstract
 
         return false;
     }
-
+    
     /**
-     * Is the request a Javascript XMLHttpRequest?
+     * Get the request URI scheme
      *
-     * Should work with Prototype/Script.aculo.us, possibly others.
-     *
-     * @return boolean
+     * @return string
      */
-    public function isXmlHttpRequest()
+    public function getScheme()
     {
-        return ($this->getHeader('X_REQUESTED_WITH') == 'XMLHttpRequest');
+        return ($this->getServer('HTTPS') == 'on') ? self::SCHEME_HTTPS : self::SCHEME_HTTP;
+    }
+    
+    /**
+     * Get the HTTP host.
+     *
+     * "Host" ":" host [ ":" port ] ; Section 3.2.2
+     * Note the HTTP Host header is not the same as the URI host.
+     * It includes the port while the URI host doesn't.
+     *
+     * @return string
+     */
+    public function getHttpHost()
+    {
+        $host = $this->getServer('HTTP_HOST');
+        if (!empty($host)) {
+            return $host;
+        }
+        
+        $scheme = $this->getScheme();
+        $name   = $this->getServer('SERVER_NAME');
+        $port   = $this->getServer('SERVER_PORT'); 
+
+        if (($scheme == self::SCHEME_HTTP && $port == 80) || ($scheme == self::SCHEME_HTTPS && $port == 443)) {
+            return $name;
+        } else {
+            return $name . ':' . $port;
+        }
     }
 }
