@@ -1,5 +1,4 @@
 <?php
-
 /**
  * Zend Framework
  *
@@ -16,9 +15,9 @@
  * @category   Zend
  * @package    Zend_Db
  * @subpackage Table
- * @copyright  Copyright (c) 2005-2007 Zend Technologies USA Inc. (http://www.zend.com)
+ * @copyright  Copyright (c) 2005-2008 Zend Technologies USA Inc. (http://www.zend.com)
  * @license    http://framework.zend.com/license/new-bsd     New BSD License
- * @version    $Id: Abstract.php 6337 2007-09-13 19:57:28Z bkarwin $
+ * @version    $Id: Abstract.php 6332 2007-09-13 00:35:08Z bkarwin $
  */
 
 /**
@@ -35,7 +34,7 @@ require_once 'Zend/Loader.php';
  * @category   Zend
  * @package    Zend_Db
  * @subpackage Table
- * @copyright  Copyright (c) 2005-2007 Zend Technologies USA Inc. (http://www.zend.com)
+ * @copyright  Copyright (c) 2005-2008 Zend Technologies USA Inc. (http://www.zend.com)
  * @license    http://framework.zend.com/license/new-bsd     New BSD License
  */
 abstract class Zend_Db_Table_Row_Abstract
@@ -59,6 +58,14 @@ abstract class Zend_Db_Table_Row_Abstract
      */
     protected $_cleanData = array();
 
+    /**
+     * Tracks columns where data has been updated. Allows more specific insert and
+     * update operations.
+     *
+     * @var array
+     */
+    protected $_modifiedFields = array();
+
     /**
      * Zend_Db_Table_Abstract parent class or instance.
      *
@@ -75,6 +82,15 @@ abstract class Zend_Db_Table_Row_Abstract
      */
     protected $_connected = true;
 
+    /**
+     * A row is marked read only if it contains columns that are not physically represented within
+     * the database schema (e.g. evaluated columns/Zend_Db_Expr columns). This can also be passed
+     * as a run-time config options as a means of protecting row data.
+     *
+     * @var boolean
+     */
+    protected $_readOnly = false;
+
     /**
      * Name of the class of the Zend_Db_Table_Abstract object.
      *
@@ -118,11 +134,17 @@ abstract class Zend_Db_Table_Row_Abstract
             $this->_cleanData = $this->_data;
         }
 
+        if (isset($config['readOnly']) && $config['readOnly'] === true) {
+            $this->setReadOnly(true);
+        }
+
         // Retrieve primary keys from table schema
-        if ($table = $this->_getTable()) {
+        if (($table = $this->_getTable())) {
             $info = $table->info();
             $this->_primary = (array) $info['primary'];
         }
+
+        $this->init();
     }
 
     /**
@@ -178,6 +200,7 @@ abstract class Zend_Db_Table_Row_Abstract
             throw new Zend_Db_Table_Row_Exception("Specified column \"$columnName\" is not in the row");
         }
         $this->_data[$columnName] = $value;
+        $this->_modifiedFields[$columnName] = true;
     }
 
     /**
@@ -199,7 +222,7 @@ abstract class Zend_Db_Table_Row_Abstract
      */
     public function __sleep()
     {
-        return array('_tableClass', '_primary', '_data', '_cleanData');
+        return array('_tableClass', '_primary', '_data', '_cleanData', '_readOnly' ,'_modifiedFields');
     }
 
     /**
@@ -214,6 +237,17 @@ abstract class Zend_Db_Table_Row_Abstract
         $this->_connected = false;
     }
 
+    /**
+     * Initialize object
+     *
+     * Called from {@link __construct()} as final step of object instantiation.
+     *
+     * @return void
+     */
+    public function init()
+    {
+    }
+
     /**
      * Returns the table object, or null if this is disconnected row
      *
@@ -232,7 +266,7 @@ abstract class Zend_Db_Table_Row_Abstract
      * @return boolean
      * @throws Zend_Db_Table_Row_Exception
      */
-    public function setTable(Zend_Db_Table_Abstract $table)
+    public function setTable(Zend_Db_Table_Abstract $table = null)
     {
         if ($table == null) {
             $this->_table = null;
@@ -277,6 +311,47 @@ abstract class Zend_Db_Table_Row_Abstract
         return $this->_tableClass;
     }
 
+    /**
+     * Test the connected status of the row.
+     *
+     * @return boolean
+     */
+    public function isConnected()
+    {
+        return $this->_connected;
+    }
+
+    /**
+     * Test the read-only status of the row.
+     *
+     * @return boolean
+     */
+    public function isReadOnly()
+    {
+        return $this->_readOnly;
+    }
+
+    /**
+     * Set the read-only status of the row.
+     *
+     * @param boolean $flag
+     * @return boolean
+     */
+    public function setReadOnly($flag)
+    {
+        $this->_readOnly = (bool) $flag;
+    }
+
+    /**
+     * Returns an instance of the parent table's Zend_Db_Table_Select object.
+     *
+     * @return Zend_Db_Table_Select
+     */
+    public function select()
+    {
+        return $this->getTable()->select();
+    }
+
     /**
      * Saves the properties to the database.
      *
@@ -306,6 +381,14 @@ abstract class Zend_Db_Table_Row_Abstract
      */
     protected function _doInsert()
     {
+        /**
+         * A read-only row cannot be saved.
+         */
+        if ($this->_readOnly === true) {
+            require_once 'Zend/Db/Table/Row/Exception.php';
+            throw new Zend_Db_Table_Row_Exception('This row has been marked read-only');
+        }
+
         /**
          * Run pre-INSERT logic
          */
@@ -314,7 +397,8 @@ abstract class Zend_Db_Table_Row_Abstract
         /**
          * Execute the INSERT (this may throw an exception)
          */
-        $primaryKey = $this->_getTable()->insert($this->_data);
+        $data = array_intersect_key($this->_data, $this->_modifiedFields);
+        $primaryKey = $this->_getTable()->insert($data);
 
         /**
          * Normalize the result to an array indexed by primary key column(s).
@@ -323,7 +407,7 @@ abstract class Zend_Db_Table_Row_Abstract
         if (is_array($primaryKey)) {
             $newPrimaryKey = $primaryKey;
         } else {
-            $newPrimaryKey = array(current((array)$this->_primary) => $primaryKey);
+            $newPrimaryKey = array(current((array) $this->_primary) => $primaryKey);
         }
 
         /**
@@ -353,6 +437,14 @@ abstract class Zend_Db_Table_Row_Abstract
      */
     protected function _doUpdate()
     {
+        /**
+         * A read-only row cannot be saved.
+         */
+        if ($this->_readOnly === true) {
+            require_once 'Zend/Db/Table/Row/Exception.php';
+            throw new Zend_Db_Table_Row_Exception('This row has been marked read-only');
+        }
+
         /**
          * Get expressions for a WHERE clause
          * based on the primary key value(s).
@@ -365,10 +457,10 @@ abstract class Zend_Db_Table_Row_Abstract
         $this->_update();
 
         /**
-         * Compare the data to the clean data to discover
+         * Compare the data to the modified fields array to discover
          * which columns have been changed.
          */
-        $diffData = array_diff_assoc($this->_data, $this->_cleanData);
+        $diffData = array_intersect_key($this->_data, $this->_modifiedFields);
 
         /**
          * Were any of the changed columns part of the primary key?
@@ -385,10 +477,9 @@ abstract class Zend_Db_Table_Row_Abstract
                 $db = $this->_getTable()->getAdapter();
                 $pkNew = $this->_getPrimaryKey(true);
                 $pkOld = $this->_getPrimaryKey(false);
-                $thisClass = get_class($this);
                 foreach ($depTables as $tableClass) {
                     try {
-                        Zend_Loader::loadClass($tableClass);
+                        @Zend_Loader::loadClass($tableClass);
                     } catch (Zend_Exception $e) {
                         require_once 'Zend/Db/Table/Row/Exception.php';
                         throw new Zend_Db_Table_Row_Exception($e->getMessage());
@@ -405,9 +496,8 @@ abstract class Zend_Db_Table_Row_Abstract
          * Use the $diffData variable, so the UPDATE statement
          * includes SET terms only for data values that changed.
          */
-        $result = 0;
         if (count($diffData) > 0) {
-            $result = $this->_getTable()->update($diffData, $where);
+            $this->_getTable()->update($diffData, $where);
         }
 
         /**
@@ -431,9 +521,9 @@ abstract class Zend_Db_Table_Row_Abstract
         $primaryKey = $this->_getPrimaryKey(true);
         if (count($primaryKey) == 1) {
             return current($primaryKey);
-        } else {
-            return $primaryKey;
         }
+
+        return $primaryKey;
     }
 
     /**
@@ -443,6 +533,14 @@ abstract class Zend_Db_Table_Row_Abstract
      */
     public function delete()
     {
+        /**
+         * A read-only row cannot be deleted.
+         */
+        if ($this->_readOnly === true) {
+            require_once 'Zend/Db/Table/Row/Exception.php';
+            throw new Zend_Db_Table_Row_Exception('This row has been marked read-only');
+        }
+        
         $where = $this->_getWhereQuery();
 
         /**
@@ -457,10 +555,9 @@ abstract class Zend_Db_Table_Row_Abstract
         if (!empty($depTables)) {
             $db = $this->_getTable()->getAdapter();
             $pk = $this->_getPrimaryKey();
-            $thisClass = get_class($this);
             foreach ($depTables as $tableClass) {
                 try {
-                    Zend_Loader::loadClass($tableClass);
+                    @Zend_Loader::loadClass($tableClass);
                 } catch (Zend_Exception $e) {
                     require_once 'Zend/Db/Table/Row/Exception.php';
                     throw new Zend_Db_Table_Row_Exception($e->getMessage());
@@ -498,7 +595,7 @@ abstract class Zend_Db_Table_Row_Abstract
      */
     public function toArray()
     {
-        return $this->_data;
+        return (array)$this->_data;
     }
 
     /**
@@ -510,12 +607,22 @@ abstract class Zend_Db_Table_Row_Abstract
     public function setFromArray(array $data)
     {
         foreach ($data as $columnName => $value) {
-            $this->$columnName = $value;
+            $this->__set($columnName, $value);
         }
 
         return $this;
     }
 
+    /**
+     * Refreshes properties from the database.
+     *
+     * @return void
+     */
+    public function refresh()
+    {
+        return $this->_refresh();
+    }
+
     /**
      * Retrieves an instance of the parent table.
      *
@@ -538,6 +645,11 @@ abstract class Zend_Db_Table_Row_Abstract
      */
     protected function _getPrimaryKey($useDirty = true)
     {
+        if (!is_array($this->_primary)) {
+            require_once 'Zend/Db/Table/Row/Exception.php';
+            throw new Zend_Db_Table_Row_Exception("The primary key must be set as an array");
+        }
+
         $primary = array_flip($this->_primary);
         if ($useDirty) {
             $array = array_intersect_key($this->_data, $primary);
@@ -567,12 +679,12 @@ abstract class Zend_Db_Table_Row_Abstract
 
         // retrieve recently updated row using primary keys
         $where = array();
-        foreach ($primaryKey as $columnName => $value) {
-            $column = $db->quoteIdentifier($columnName, true);
-            $type = $metadata[$columnName]['DATA_TYPE'];
-            $where[] = $db->quoteInto("$column = ?", $value, $type);
+        foreach ($primaryKey as $column => $value) {
+            $tableName = $db->quoteIdentifier($info[Zend_Db_Table_Abstract::NAME], true);
+            $type = $metadata[$column]['DATA_TYPE'];
+            $columnName = $db->quoteIdentifier($column, true);
+            $where[] = $db->quoteInto("{$tableName}.{$columnName} = ?", $value, $type);
         }
-
         return $where;
     }
 
@@ -593,6 +705,7 @@ abstract class Zend_Db_Table_Row_Abstract
 
         $this->_data = $row->toArray();
         $this->_cleanData = $this->_data;
+        $this->_modifiedFields = array();
     }
 
     /**
@@ -685,16 +798,17 @@ abstract class Zend_Db_Table_Row_Abstract
      *
      * @param string|Zend_Db_Table_Abstract  $dependentTable
      * @param string                         OPTIONAL $ruleKey
+     * @param Zend_Db_Table_Select           OPTIONAL $select
      * @return Zend_Db_Table_Rowset_Abstract Query result from $dependentTable
      * @throws Zend_Db_Table_Row_Exception If $dependentTable is not a table or is not loadable.
      */
-    public function findDependentRowset($dependentTable, $ruleKey = null)
+    public function findDependentRowset($dependentTable, $ruleKey = null, Zend_Db_Table_Select $select = null)
     {
         $db = $this->_getTable()->getAdapter();
 
         if (is_string($dependentTable)) {
             try {
-                Zend_Loader::loadClass($dependentTable);
+                @Zend_Loader::loadClass($dependentTable);
             } catch (Zend_Exception $e) {
                 require_once 'Zend/Db/Table/Row/Exception.php';
                 throw new Zend_Db_Table_Row_Exception($e->getMessage());
@@ -710,19 +824,27 @@ abstract class Zend_Db_Table_Row_Abstract
             throw new Zend_Db_Table_Row_Exception("Dependent table must be a Zend_Db_Table_Abstract, but it is $type");
         }
 
+        if ($select === null) {
+            $select = $dependentTable->select();
+        } else {
+            $select->setTable($dependentTable);
+        }
+
         $map = $this->_prepareReference($dependentTable, $this->_getTable(), $ruleKey);
 
-        $where = array();
         for ($i = 0; $i < count($map[Zend_Db_Table_Abstract::COLUMNS]); ++$i) {
             $parentColumnName = $db->foldCase($map[Zend_Db_Table_Abstract::REF_COLUMNS][$i]);
             $value = $this->_data[$parentColumnName];
-            $dependentColumnName = $db->foldCase($map[Zend_Db_Table_Abstract::COLUMNS][$i]);
-            $dependentColumn = $db->quoteIdentifier($dependentColumnName, true);
+            // Use adapter from dependent table to ensure correct query construction
+            $dependentDb = $dependentTable->getAdapter();
+            $dependentColumnName = $dependentDb->foldCase($map[Zend_Db_Table_Abstract::COLUMNS][$i]);
+            $dependentColumn = $dependentDb->quoteIdentifier($dependentColumnName, true);
             $dependentInfo = $dependentTable->info();
             $type = $dependentInfo[Zend_Db_Table_Abstract::METADATA][$dependentColumnName]['DATA_TYPE'];
-            $where[] = $db->quoteInto("$dependentColumn = ?", $value, $type);
+            $select->where("$dependentColumn = ?", $value, $type);
         }
-        return $dependentTable->fetchAll($where);
+
+        return $dependentTable->fetchAll($select);
     }
 
     /**
@@ -733,13 +855,13 @@ abstract class Zend_Db_Table_Row_Abstract
      * @return Zend_Db_Table_Row_Abstract   Query result from $parentTable
      * @throws Zend_Db_Table_Row_Exception If $parentTable is not a table or is not loadable.
      */
-    public function findParentRow($parentTable, $ruleKey = null)
+    public function findParentRow($parentTable, $ruleKey = null, Zend_Db_Table_Select $select = null)
     {
         $db = $this->_getTable()->getAdapter();
 
         if (is_string($parentTable)) {
             try {
-                Zend_Loader::loadClass($parentTable);
+                @Zend_Loader::loadClass($parentTable);
             } catch (Zend_Exception $e) {
                 require_once 'Zend/Db/Table/Row/Exception.php';
                 throw new Zend_Db_Table_Row_Exception($e->getMessage());
@@ -755,19 +877,27 @@ abstract class Zend_Db_Table_Row_Abstract
             throw new Zend_Db_Table_Row_Exception("Parent table must be a Zend_Db_Table_Abstract, but it is $type");
         }
 
+        if ($select === null) {
+            $select = $parentTable->select();
+        } else {
+            $select->setTable($parentTable);
+        }
+
         $map = $this->_prepareReference($this->_getTable(), $parentTable, $ruleKey);
 
-        $where = array();
         for ($i = 0; $i < count($map[Zend_Db_Table_Abstract::COLUMNS]); ++$i) {
             $dependentColumnName = $db->foldCase($map[Zend_Db_Table_Abstract::COLUMNS][$i]);
             $value = $this->_data[$dependentColumnName];
-            $parentColumnName = $db->foldCase($map[Zend_Db_Table_Abstract::REF_COLUMNS][$i]);
-            $parentColumn = $db->quoteIdentifier($parentColumnName, true);
+            // Use adapter from parent table to ensure correct query construction
+            $parentDb = $parentTable->getAdapter();
+            $parentColumnName = $parentDb->foldCase($map[Zend_Db_Table_Abstract::REF_COLUMNS][$i]);
+            $parentColumn = $parentDb->quoteIdentifier($parentColumnName, true);
             $parentInfo = $parentTable->info();
             $type = $parentInfo[Zend_Db_Table_Abstract::METADATA][$parentColumnName]['DATA_TYPE'];
-            $where[] = $db->quoteInto("$parentColumn = ?", $value, $type);
+            $select->where("$parentColumn = ?", $value, $type);
         }
-        return $parentTable->fetchRow($where);
+
+        return $parentTable->fetchRow($select);
     }
 
     /**
@@ -778,13 +908,14 @@ abstract class Zend_Db_Table_Row_Abstract
      * @return Zend_Db_Table_Rowset_Abstract Query result from $matchTable
      * @throws Zend_Db_Table_Row_Exception If $matchTable or $intersectionTable is not a table class or is not loadable.
      */
-    public function findManyToManyRowset($matchTable, $intersectionTable, $callerRefRule = null, $matchRefRule = null)
+    public function findManyToManyRowset($matchTable, $intersectionTable, $callerRefRule = null,
+                                         $matchRefRule = null, Zend_Db_Table_Select $select = null)
     {
         $db = $this->_getTable()->getAdapter();
 
         if (is_string($intersectionTable)) {
             try {
-                Zend_Loader::loadClass($intersectionTable);
+                @Zend_Loader::loadClass($intersectionTable);
             } catch (Zend_Exception $e) {
                 require_once 'Zend/Db/Table/Row/Exception.php';
                 throw new Zend_Db_Table_Row_Exception($e->getMessage());
@@ -802,7 +933,7 @@ abstract class Zend_Db_Table_Row_Abstract
 
         if (is_string($matchTable)) {
             try {
-                Zend_Loader::loadClass($matchTable);
+                @Zend_Loader::loadClass($matchTable);
             } catch (Zend_Exception $e) {
                 require_once 'Zend/Db/Table/Row/Exception.php';
                 throw new Zend_Db_Table_Row_Exception($e->getMessage());
@@ -818,7 +949,15 @@ abstract class Zend_Db_Table_Row_Abstract
             throw new Zend_Db_Table_Row_Exception("Match table must be a Zend_Db_Table_Abstract, but it is $type");
         }
 
+        if ($select === null) {
+            $select = $matchTable->select();
+        } else {
+            $select->setTable($matchTable);
+        }
+
+        // Use adapter from intersection table to ensure correct query construction
         $interInfo = $intersectionTable->info();
+        $interDb   = $intersectionTable->getAdapter();
         $interName = $interInfo['name'];
         $matchInfo = $matchTable->info();
         $matchName = $matchInfo['name'];
@@ -826,40 +965,42 @@ abstract class Zend_Db_Table_Row_Abstract
         $matchMap = $this->_prepareReference($intersectionTable, $matchTable, $matchRefRule);
 
         for ($i = 0; $i < count($matchMap[Zend_Db_Table_Abstract::COLUMNS]); ++$i) {
-            $interCol = $db->quoteIdentifier('i' . '.' . $matchMap[Zend_Db_Table_Abstract::COLUMNS][$i], true);
-            $matchCol = $db->quoteIdentifier('m' . '.' . $matchMap[Zend_Db_Table_Abstract::REF_COLUMNS][$i], true);
+            $interCol = $interDb->quoteIdentifier('i' . '.' . $matchMap[Zend_Db_Table_Abstract::COLUMNS][$i], true);
+            $matchCol = $interDb->quoteIdentifier('m' . '.' . $matchMap[Zend_Db_Table_Abstract::REF_COLUMNS][$i], true);
             $joinCond[] = "$interCol = $matchCol";
         }
         $joinCond = implode(' AND ', $joinCond);
 
-        $select = $db->select()
-            ->from(array('i' => $interName), array())
-            ->join(array('m' => $matchName), $joinCond, '*');
+        $select->from(array('i' => $interName))
+               ->joinInner(array('m' => $matchName), $joinCond)
+               ->setIntegrityCheck(false);
 
         $callerMap = $this->_prepareReference($intersectionTable, $this->_getTable(), $callerRefRule);
 
         for ($i = 0; $i < count($callerMap[Zend_Db_Table_Abstract::COLUMNS]); ++$i) {
             $callerColumnName = $db->foldCase($callerMap[Zend_Db_Table_Abstract::REF_COLUMNS][$i]);
             $value = $this->_data[$callerColumnName];
-            $interColumnName = $db->foldCase($callerMap[Zend_Db_Table_Abstract::COLUMNS][$i]);
-            $interCol = $db->quoteIdentifier("i.$interColumnName", true);
-            $matchColumnName = $db->foldCase($matchMap[Zend_Db_Table_Abstract::REF_COLUMNS][$i]);
-            $matchInfo = $matchTable->info();
-            $type = $matchInfo[Zend_Db_Table_Abstract::METADATA][$matchColumnName]['DATA_TYPE'];
-            $select->where($db->quoteInto("$interCol = ?", $value, $type));
+            $interColumnName = $interDb->foldCase($callerMap[Zend_Db_Table_Abstract::COLUMNS][$i]);
+            $interCol = $interDb->quoteIdentifier("i.$interColumnName", true);
+            $matchColumnName = $interDb->foldCase($matchMap[Zend_Db_Table_Abstract::REF_COLUMNS][$i]);
+            $interInfo = $intersectionTable->info();
+            $type = $interInfo[Zend_Db_Table_Abstract::METADATA][$interColumnName]['DATA_TYPE'];
+            $select->where($interDb->quoteInto("$interCol = ?", $value, $type));
         }
+
         $stmt = $select->query();
 
         $config = array(
             'table'    => $matchTable,
             'data'     => $stmt->fetchAll(Zend_Db::FETCH_ASSOC),
             'rowClass' => $matchTable->getRowClass(),
+            'readOnly' => false,
             'stored'   => true
         );
 
         $rowsetClass = $matchTable->getRowsetClass();
         try {
-            Zend_Loader::loadClass($rowsetClass);
+            @Zend_Loader::loadClass($rowsetClass);
         } catch (Zend_Exception $e) {
             require_once 'Zend/Db/Table/Row/Exception.php';
             throw new Zend_Db_Table_Row_Exception($e->getMessage());
@@ -873,12 +1014,19 @@ abstract class Zend_Db_Table_Row_Abstract
      * to the above methods.
      *
      * @param string $method
-     * @param array $args
+     * @param array $args OPTIONAL Zend_Db_Table_Select query modifier
      * @return Zend_Db_Table_Row_Abstract|Zend_Db_Table_Rowset_Abstract
      * @throws Zend_Db_Table_Row_Exception If an invalid method is called.
      */
-    protected function __call($method, array $args)
+    public function __call($method, array $args)
     {
+        $matches = array();
+
+        if (count($args) && $args[0] instanceof Zend_Db_Table_Select) {
+            $select = $args[0];
+        } else {
+            $select = null;
+        }
 
         /**
          * Recognize methods for Has-Many cases:
@@ -889,7 +1037,7 @@ abstract class Zend_Db_Table_Row_Abstract
         if (preg_match('/^findParent(\w+?)(?:By(\w+))?$/', $method, $matches)) {
             $class    = $matches[1];
             $ruleKey1 = isset($matches[2]) ? $matches[2] : null;
-            return $this->findParentRow($class, $ruleKey1);
+            return $this->findParentRow($class, $ruleKey1, $select);
         }
 
         /**
@@ -904,7 +1052,7 @@ abstract class Zend_Db_Table_Row_Abstract
             $viaClass = $matches[2];
             $ruleKey1 = isset($matches[3]) ? $matches[3] : null;
             $ruleKey2 = isset($matches[4]) ? $matches[4] : null;
-            return $this->findManyToManyRowset($class, $viaClass, $ruleKey1, $ruleKey2);
+            return $this->findManyToManyRowset($class, $viaClass, $ruleKey1, $ruleKey2, $select);
         }
 
         /**
@@ -916,7 +1064,7 @@ abstract class Zend_Db_Table_Row_Abstract
         if (preg_match('/^find(\w+?)(?:By(\w+))?$/', $method, $matches)) {
             $class    = $matches[1];
             $ruleKey1 = isset($matches[2]) ? $matches[2] : null;
-            return $this->findDependentRowset($class, $ruleKey1);
+            return $this->findDependentRowset($class, $ruleKey1, $select);
         }
 
         require_once 'Zend/Db/Table/Row/Exception.php';
