@@ -13,7 +13,7 @@
  * to license@zend.com so we can send you a copy immediately.
  *
  * @package    Zend_Pdf
- * @copyright  Copyright (c) 2005-2007 Zend Technologies USA Inc. (http://www.zend.com)
+ * @copyright  Copyright (c) 2005-2008 Zend Technologies USA Inc. (http://www.zend.com)
  * @license    http://framework.zend.com/license/new-bsd     New BSD License
  */
 
@@ -52,7 +52,7 @@ require_once 'Zend/Pdf/Color/Cmyk.php';
  * PDF Page
  *
  * @package    Zend_Pdf
- * @copyright  Copyright (c) 2005-2007 Zend Technologies USA Inc. (http://www.zend.com)
+ * @copyright  Copyright (c) 2005-2008 Zend Technologies USA Inc. (http://www.zend.com)
  * @license    http://framework.zend.com/license/new-bsd     New BSD License
  */
 class Zend_Pdf_Page
@@ -166,6 +166,17 @@ class Zend_Pdf_Page
      */
     private $_saveCount = 0;
 
+    /**
+     * Safe Graphics State semafore
+     * 
+     * If it's false, than we can't be sure Graphics State is restored withing 
+     * context of previous contents stream (ex. drawing coordinate system may be rotated).
+     * We should encompass existing content with save/restore GS operators
+     * 
+     * @var boolean 
+     */
+    private $_safeGS;
+
     /**
      * Current font
      *
@@ -229,6 +240,8 @@ class Zend_Pdf_Page
             $this->_pageDictionary = $param1;
             $this->_objFactory     = $param2;
             $this->_attached       = true;
+            $this->_safeGS         = false;
+
             return;
 
         } else if ($param1 instanceof Zend_Pdf_Page && $param2 === null && $param3 === null) {
@@ -236,8 +249,9 @@ class Zend_Pdf_Page
             // Let already existing content and resources to be shared between pages
             // We don't give existing content modification functionality, so we don't need "deep copy"
             $this->_objFactory = $param1->_objFactory;
-            $this->_attached = &$param1->_attached;
-
+            $this->_attached   = &$param1->_attached;
+            $this->_safeGS     = false;
+            
             $this->_pageDictionary = $this->_objFactory->newObject(new Zend_Pdf_Element_Dictionary());
 
             foreach ($param1->_pageDictionary->getKeys() as $key) {
@@ -265,7 +279,8 @@ class Zend_Pdf_Page
                    ($param2 === null || $param2 instanceof Zend_Pdf_ElementFactory_Interface) &&
                    $param3 === null) {
             $this->_objFactory = ($param2 !== null)? $param2 : Zend_Pdf_ElementFactory::createFactory(1);
-            $this->_attached = false;
+            $this->_attached   = false;
+            $this->_safeGS     = true; /** New page created. That's users App responsibility to track GS changes */
 
             switch (strtolower($param1)) {
                 case 'a4':
@@ -303,6 +318,7 @@ class Zend_Pdf_Page
                    ($param3 === null || $param3 instanceof Zend_Pdf_ElementFactory_Interface)) {
             $this->_objFactory = ($param3 !== null)? $param3 : Zend_Pdf_ElementFactory::createFactory(1);
             $this->_attached = false;
+            $this->_safeGS   = true; /** New page created. That's users App responsibility to track GS changes */
             $pageWidth  = $param1;
             $pageHeight = $param2;
 
@@ -368,7 +384,7 @@ class Zend_Pdf_Page
 
         return $newResName;
     }
-
+    
     /**
      * Add procedureSet to the Page description
      *
@@ -435,6 +451,27 @@ class Zend_Pdf_Page
             $this->_pageDictionary->Contents->touch();
         }
 
+        if ((!$this->_safeGS)  &&  (count($this->_pageDictionary->Contents->items) != 0)) {
+        	/**
+        	 * Page already has some content which is not treated as safe.
+        	 * 
+        	 * Add save/restore GS operators
+        	 */
+            $this->_addProcSet('PDF');
+        	
+        	$newContentsArray = new Zend_Pdf_Element_Array();
+        	$newContentsArray->items[] = $this->_objFactory->newStreamObject(" q\n");
+        	foreach ($this->_pageDictionary->Contents->items as $contentStream) {
+        		$newContentsArray->items[] = $contentStream;
+        	}
+            $newContentsArray->items[] = $this->_objFactory->newStreamObject(" Q\n");
+
+        	$this->_pageDictionary->touch();
+        	$this->_pageDictionary->Contents = $newContentsArray;
+        	
+        	$this->_safeGS = true;
+        }
+        
         $this->_pageDictionary->Contents->items[] =
                 $this->_objFactory->newStreamObject($this->_contents);
 
@@ -576,6 +613,61 @@ class Zend_Pdf_Page
         $this->_contents .= $style->instructions($this->_pageDictionary->Resources);
     }
 
+    /**
+     * Set the transparancy
+     *
+     * $alpha == 0  - transparent
+     * $alpha == 1  - opaque
+     *
+     * Transparency modes, supported by PDF:
+     * Normal (default), Multiply, Screen, Overlay, Darken, Lighten, ColorDodge, ColorBurn, HardLight,
+     * SoftLight, Difference, Exclusion
+     *
+     * @param float $alpha
+     * @param string $mode
+     * @throws Zend_Pdf_Exception
+     */
+    public function setAlpha($alpha, $mode = 'Normal')
+    {
+    	if (!in_array($mode, array('Normal', 'Multiply', 'Screen', 'Overlay', 'Darken', 'Lighten', 'ColorDodge',
+    	                           'ColorBurn', 'HardLight', 'SoftLight', 'Difference', 'Exclusion'))) {
+    	   throw new Zend_Pdf_Exception('Unsupported transparency mode.');
+    	}
+    	if (!is_numeric($alpha)  ||  $alpha < 0  ||  $alpha > 1) {
+    		throw new Zend_Pdf_Exception('Alpha value must be numeric between 0 (transparent) and 1 (opaque).');
+    	}
+
+        $this->_addProcSet('Text');
+        $this->_addProcSet('PDF');
+
+        $resources = $this->_pageDictionary->Resources;
+
+        // Check if Resources dictionary contains ExtGState entry
+        if ($resources->ExtGState === null) {
+            $resources->touch();
+            $resources->ExtGState = new Zend_Pdf_Element_Dictionary();
+        } else {
+            $resources->ExtGState->touch();
+        }
+
+        $idCounter = 1;
+        do {
+            $gStateName = 'GS' . $idCounter++;
+        } while ($resources->ExtGState->$gStateName !== null);
+
+
+        $gStateDictionary = new Zend_Pdf_Element_Dictionary();
+        $gStateDictionary->Type = new Zend_Pdf_Element_Name('ExtGState');
+        $gStateDictionary->BM   = new Zend_Pdf_Element_Name($mode);
+        $gStateDictionary->CA   = new Zend_Pdf_Element_Numeric($alpha);
+        $gStateDictionary->ca   = new Zend_Pdf_Element_Numeric($alpha);
+
+        $resources->ExtGState->$gStateName = $this->_objFactory->newObject($gStateDictionary);
+
+        $gStateNameObj = new Zend_Pdf_Element_Name($gStateName);
+        $this->_contents .= $gStateNameObj->toString() . " gs\n";
+    }
+
 
     /**
      * Get current font.
@@ -587,6 +679,112 @@ class Zend_Pdf_Page
         return $this->_font;
     }
 
+    /**
+     * Extract resources attached to the page
+     *
+     * This method is not intended to be used in userland, but helps to optimize some document wide operations
+     * 
+     * returns array of Zend_Pdf_Element_Dictionary objects
+     * 
+     * @internal 
+     * @return array
+     */
+    public function extractResources()
+    {
+        return $this->_pageDictionary->Resources;
+    }
+    
+    /**
+     * Extract fonts attached to the page
+     *
+     * returns array of Zend_Pdf_Resource_Font_Extracted objects
+     * 
+     * @return array
+     */
+    public function extractFonts()
+    {
+        if ($this->_pageDictionary->Resources->Font === null) {
+            // Page doesn't have any font attached
+            // Return empty array
+            return array();
+        }
+        
+        $fontResources = $this->_pageDictionary->Resources->Font;
+
+        $fontResourcesUnique = array();
+        foreach ($fontResources->getKeys() as $fontResourceName) {
+            $fontDictionary = $fontResources->$fontResourceName;
+
+            if (! ($fontDictionary instanceof Zend_Pdf_Element_Reference  ||
+                   $fontDictionary instanceof Zend_Pdf_Element_Object) ) {
+                // Font dictionary has to be an indirect object or object reference
+                continue;
+            }
+
+            $fontResourcesUnique[$fontDictionary->toString($this->_objFactory)] = $fontDictionary;
+        }
+        
+        $fonts = array();
+        foreach ($fontResourcesUnique as $resourceReference => $fontDictionary) {
+            try {
+                // Try to extract font
+                $extractedFont = new Zend_Pdf_Resource_Font_Extracted($fontDictionary);
+
+                $fonts[$resourceReference] = $extractedFont; 
+            } catch (Zend_Pdf_Exception $e) {
+                if ($e->getMessage() != 'Unsupported font type.') {
+                    throw $e;
+                }
+            }
+        }
+        
+        return $fonts;
+    } 
+
+    /**
+     * Extract font attached to the page by specific font name
+     * 
+     * $fontName should be specified in UTF-8 encoding
+     *
+     * @return Zend_Pdf_Resource_Font_Extracted|null
+     */
+    public function extractFont($fontName)
+    {
+        if ($this->_pageDictionary->Resources->Font === null) {
+            // Page doesn't have any font attached
+            return null;
+        }
+        
+        $fontResources = $this->_pageDictionary->Resources->Font;
+
+        foreach ($fontResources->getKeys() as $fontResourceName) {
+            $fontDictionary = $fontResources->$fontResourceName;
+            
+            if (! ($fontDictionary instanceof Zend_Pdf_Element_Reference  ||
+                   $fontDictionary instanceof Zend_Pdf_Element_Object) ) {
+                // Font dictionary has to be an indirect object or object reference
+                continue;
+            }
+            
+            if ($fontDictionary->BaseFont->value != $fontName) {
+                continue;
+            }
+            
+            try {
+                // Try to extract font
+                return new Zend_Pdf_Resource_Font_Extracted($fontDictionary); 
+            } catch (Zend_Pdf_Exception $e) {
+                if ($e->getMessage() != 'Unsupported font type.') {
+                    throw $e;
+                }
+                
+                // Continue searhing font with specified name
+            }
+        }
+        
+        return null;
+    } 
+
     /**
      * Get current font size
      *
