@@ -14,7 +14,7 @@
  *
  * @category   Zend
  * @package    Zend_Search_Lucene
- * @copyright  Copyright (c) 2005-2007 Zend Technologies USA Inc. (http://www.zend.com)
+ * @copyright  Copyright (c) 2005-2008 Zend Technologies USA Inc. (http://www.zend.com)
  * @license    http://framework.zend.com/license/new-bsd     New BSD License
  */
 
@@ -28,7 +28,7 @@ require_once 'Zend/Search/Lucene/Document.php';
 /** Zend_Search_Lucene_Document_Html */
 require_once 'Zend/Search/Lucene/Document/Html.php';
 
-/** Zend_Search_Lucene_Storage_Directory */
+/** Zend_Search_Lucene_Storage_Directory_Filesystem */
 require_once 'Zend/Search/Lucene/Storage/Directory/Filesystem.php';
 
 /** Zend_Search_Lucene_Storage_File_Memory */
@@ -61,6 +61,10 @@ require_once 'Zend/Search/Lucene/Search/Similarity.php';
 /** Zend_Search_Lucene_Index_SegmentInfoPriorityQueue */
 require_once 'Zend/Search/Lucene/Index/SegmentInfoPriorityQueue.php';
 
+/** Zend_Search_Lucene_LockManager */
+require_once 'Zend/Search/Lucene/LockManager.php';
+
+
 
 /** Zend_Search_Lucene_Interface */
 require_once 'Zend/Search/Lucene/Interface.php';
@@ -72,7 +76,7 @@ require_once 'Zend/Search/Lucene/Proxy.php';
 /**
  * @category   Zend
  * @package    Zend_Search_Lucene
- * @copyright  Copyright (c) 2005-2007 Zend Technologies USA Inc. (http://www.zend.com)
+ * @copyright  Copyright (c) 2005-2008 Zend Technologies USA Inc. (http://www.zend.com)
  * @license    http://framework.zend.com/license/new-bsd     New BSD License
  */
 class Zend_Search_Lucene implements Zend_Search_Lucene_Interface
@@ -138,13 +142,6 @@ class Zend_Search_Lucene implements Zend_Search_Lucene_Interface
     private $_hasChanges = false;
 
 
-    /**
-     * Index lock object
-     *
-     * @var Zend_Search_Lucene_Storage_File
-     */
-    private $_lock;
-
     /**
      * Signal, that index is already closed, changes are fixed and resources are cleaned up
      *
@@ -159,6 +156,23 @@ class Zend_Search_Lucene implements Zend_Search_Lucene_Interface
      */
     private $_refCount = 0;
 
+    /**
+     * Current segment generation
+     *
+     * @var integer
+     */
+    private $_generation;
+
+    const FORMAT_PRE_2_1 = 0;
+    const FORMAT_2_1     = 1;
+    const FORMAT_2_3     = 2;
+
+    /**
+     * Index format version
+     *
+     * @var integer
+     */
+    private $_formatVersion;
 
     /**
      * Create index
@@ -182,56 +196,131 @@ class Zend_Search_Lucene implements Zend_Search_Lucene_Interface
         return new Zend_Search_Lucene_Proxy(new Zend_Search_Lucene($directory, false));
     }
 
+    /** Generation retrieving counter */
+    const GENERATION_RETRIEVE_COUNT = 10;
+
+    /** Pause between generation retrieving attempts in milliseconds */
+    const GENERATION_RETRIEVE_PAUSE = 50;
+
     /**
-     * Opens the index.
+     * Get current generation number
      *
-     * IndexReader constructor needs Directory as a parameter. It should be
-     * a string with a path to the index folder or a Directory object.
+     * Returns generation number
+     * 0 means pre-2.1 index format
+     * -1 means there are no segments files.
      *
-     * @param mixed $directory
+     * @param Zend_Search_Lucene_Storage_Directory $directory
+     * @return integer
      * @throws Zend_Search_Lucene_Exception
      */
-    public function __construct($directory = null, $create = false)
+    public static function getActualGeneration(Zend_Search_Lucene_Storage_Directory $directory)
     {
-        if ($directory === null) {
-            throw new Zend_Search_Exception('No index directory specified');
-        }
+        /**
+         * Zend_Search_Lucene uses segments.gen file to retrieve current generation number
+         *
+         * Apache Lucene index format documentation mentions this method only as a fallback method
+         *
+         * Nevertheless we use it according to the performance considerations
+         *
+         * @todo check if we can use some modification of Apache Lucene generation determination algorithm
+         *       without performance problems
+         */
 
-        if ($directory instanceof Zend_Search_Lucene_Storage_Directory_Filesystem) {
-            $this->_directory      = $directory;
-            $this->_closeDirOnExit = false;
-        } else {
-            $this->_directory      = new Zend_Search_Lucene_Storage_Directory_Filesystem($directory);
-            $this->_closeDirOnExit = true;
-        }
+        try {
+            for ($count = 0; $count < self::GENERATION_RETRIEVE_COUNT; $count++) {
+                // Try to get generation file
+                $genFile = $directory->getFileObject('segments.gen', false);
 
+                $format = $genFile->readInt();
+                if ($format != (int)0xFFFFFFFE) {
+                    throw new Zend_Search_Lucene_Exception('Wrong segments.gen file format');
+                }
 
-        // Get a shared lock to the index
-        $this->_lock = $this->_directory->createFile('index.lock');
+                $gen1 = $genFile->readLong();
+                $gen2 = $genFile->readLong();
 
-        $this->_segmentInfos = array();
+                if ($gen1 == $gen2) {
+                    return $gen1;
+                }
 
-        if ($create) {
-            // Throw an exception if index is under processing now
-            if (!$this->_lock->lock(LOCK_EX, true)) {
-                throw new Zend_Search_Lucene_Exception('Can\'t create index. It\'s under processing now');
+                usleep(self::GENERATION_RETRIEVE_PAUSE * 1000);
             }
 
-            // Writer will create segments file for empty segments list
-            $this->_writer = new Zend_Search_Lucene_Index_Writer($this->_directory, $this->_segmentInfos, true);
+            // All passes are failed
+            throw new Zend_Search_Lucene_Exception('Index is under processing now');
+        } catch (Zend_Search_Lucene_Exception $e) {
+            if (strpos($e->getMessage(), 'is not readable') !== false) {
+                try {
+                    // Try to open old style segments file
+                    $segmentsFile = $directory->getFileObject('segments', false);
 
-            if (!$this->_lock->lock(LOCK_SH)) {
-                throw new Zend_Search_Lucene_Exception('Can\'t reduce lock level from Exclusive to Shared');
+                    // It's pre-2.1 index
+                    return 0;
+                } catch (Zend_Search_Lucene_Exception $e) {
+                    if (strpos($e->getMessage(), 'is not readable') !== false) {
+                        return -1;
+                    } else {
+                        throw $e;
+                    }
+                }
+            } else {
+                throw $e;
             }
-        } else {
-            // Wait if index is under switching from one set of segments to another (Index_Writer::_updateSegments())
-            if (!$this->_lock->lock(LOCK_SH)) {
-                throw new Zend_Search_Lucene_Exception('Can\'t obtain shared index lock');
-            }
-            $this->_writer = null;
         }
 
+        return -1;
+    }
 
+    /**
+     * Get segments file name
+     *
+     * @param integer $generation
+     * @return string
+     */
+    public static function getSegmentFileName($generation)
+    {
+        if ($generation == 0) {
+            return 'segments';
+        }
+
+        return 'segments_' . base_convert($generation, 10, 36);
+    }
+
+    /**
+     * Get index format version
+     *
+     * @return integer
+     */
+    public function getFormatVersion()
+    {
+        return $this->_formatVersion;
+    }
+
+    /**
+     * Set index format version.
+     * Index is converted to this format at the nearest upfdate time
+     *
+     * @param int $formatVersion
+     * @throws Zend_Search_Lucene_Exception
+     */
+    public function setFormatVersion($formatVersion)
+    {
+    	if ($formatVersion != self::FORMAT_PRE_2_1  &&
+    	    $formatVersion != self::FORMAT_2_1  &&
+            $formatVersion != self::FORMAT_2_3) {
+    		throw new Zend_Search_Lucene_Exception('Unsupported index format');
+    	}
+
+    	$this->_formatVersion = $formatVersion;
+    }
+
+    /**
+     * Read segments file for pre-2.1 Lucene index format
+     *
+     * @throws Zend_Search_Lucene_Exception
+     */
+    private function _readPre21SegmentsFile()
+    {
         $segmentsFile = $this->_directory->getFileObject('segments');
 
         $format = $segmentsFile->readInt();
@@ -257,10 +346,183 @@ class Zend_Search_Lucene implements Zend_Search_Lucene_Interface
             $segSize = $segmentsFile->readInt();
             $this->_docCount += $segSize;
 
-            $this->_segmentInfos[] =
-                                new Zend_Search_Lucene_Index_SegmentInfo($segName,
+            $this->_segmentInfos[$segName] =
+                                new Zend_Search_Lucene_Index_SegmentInfo($this->_directory,
+                                                                         $segName,
+                                                                         $segSize);
+        }
+
+        // Use 2.1 as a target version. Index will be reorganized at update time.
+        $this->_formatVersion = self::FORMAT_2_1;
+    }
+
+    /**
+     * Read segments file
+     *
+     * @throws Zend_Search_Lucene_Exception
+     */
+    private function _readSegmentsFile()
+    {
+        $segmentsFile = $this->_directory->getFileObject(self::getSegmentFileName($this->_generation));
+
+        $format = $segmentsFile->readInt();
+
+        if ($format == (int)0xFFFFFFFC) {
+            $this->_formatVersion = self::FORMAT_2_3;
+        } else if ($format == (int)0xFFFFFFFD) {
+            $this->_formatVersion = self::FORMAT_2_1;
+        } else {
+            throw new Zend_Search_Lucene_Exception('Unsupported segments file format');
+        }
+
+        // read version
+        // $segmentsFile->readLong();
+        $segmentsFile->readInt(); $segmentsFile->readInt();
+
+        // read segment name counter
+        $segmentsFile->readInt();
+
+        $segments = $segmentsFile->readInt();
+
+        $this->_docCount = 0;
+
+        // read segmentInfos
+        for ($count = 0; $count < $segments; $count++) {
+            $segName = $segmentsFile->readString();
+            $segSize = $segmentsFile->readInt();
+
+            // 2.1+ specific properties
+            //$delGen          = $segmentsFile->readLong();
+            $delGenHigh        = $segmentsFile->readInt();
+            $delGenLow         = $segmentsFile->readInt();
+            if ($delGenHigh == (int)0xFFFFFFFF  && $delGenLow == (int)0xFFFFFFFF) {
+                $delGen = -1; // There are no deletes
+            } else {
+                $delGen = ($delGenHigh << 32) | $delGenLow;
+            }
+
+            if ($this->_formatVersion == self::FORMAT_2_3) {
+            	$docStoreOffset = $segmentsFile->readInt();
+
+            	if ($docStoreOffset != -1) {
+            		$docStoreSegment        = $segmentsFile->readString();
+            		$docStoreIsCompoundFile = $segmentsFile->readByte();
+
+            		$docStoreOptions = array('offset'     => $docStoreOffset,
+            		                         'segment'    => $docStoreSegment,
+            		                         'isCompound' => ($docStoreIsCompoundFile == 1));
+            	} else {
+                    $docStoreOptions = null;
+            	}
+            } else {
+                $docStoreOptions = null;
+            }
+
+            $hasSingleNormFile = $segmentsFile->readByte();
+            $numField          = $segmentsFile->readInt();
+
+            $normGens = array();
+            if ($numField != (int)0xFFFFFFFF) {
+                for ($count1 = 0; $count1 < $numField; $count1++) {
+                    $normGens[] = $segmentsFile->readLong();
+                }
+
+                throw new Zend_Search_Lucene_Exception('Separate norm files are not supported. Optimize index to use it with Zend_Search_Lucene.');
+            }
+
+            $isCompoundByte     = $segmentsFile->readByte();
+
+            if ($isCompoundByte == 0xFF) {
+            	// The segment is not a compound file
+            	$isCompound = false;
+            } else if ($isCompoundByte == 0x00) {
+                // The status is unknown
+                $isCompound = null;
+            } else if ($isCompoundByte == 0x01) {
+                // The segment is a compound file
+                $isCompound = true;
+            }
+
+            $this->_docCount += $segSize;
+
+            $this->_segmentInfos[$segName] =
+                                new Zend_Search_Lucene_Index_SegmentInfo($this->_directory,
+                                                                         $segName,
                                                                          $segSize,
-                                                                         $this->_directory);
+                                                                         $delGen,
+                                                                         $docStoreOptions,
+                                                                         $hasSingleNormFile,
+                                                                         $isCompound);
+        }
+    }
+
+    /**
+     * Opens the index.
+     *
+     * IndexReader constructor needs Directory as a parameter. It should be
+     * a string with a path to the index folder or a Directory object.
+     *
+     * @param mixed $directory
+     * @throws Zend_Search_Lucene_Exception
+     */
+    public function __construct($directory = null, $create = false)
+    {
+        if ($directory === null) {
+            throw new Zend_Search_Exception('No index directory specified');
+        }
+
+        if ($directory instanceof Zend_Search_Lucene_Storage_Directory_Filesystem) {
+            $this->_directory      = $directory;
+            $this->_closeDirOnExit = false;
+        } else {
+            $this->_directory      = new Zend_Search_Lucene_Storage_Directory_Filesystem($directory);
+            $this->_closeDirOnExit = true;
+        }
+
+        $this->_segmentInfos = array();
+
+        // Mark index as "under processing" to prevent other processes from premature index cleaning
+        Zend_Search_Lucene_LockManager::obtainReadLock($this->_directory);
+
+        $this->_generation = self::getActualGeneration($this->_directory);
+
+        if ($create) {
+        	try {
+        		Zend_Search_Lucene_LockManager::obtainWriteLock($this->_directory);
+        	} catch (Zend_Search_Lucene_Exception $e) {
+        		Zend_Search_Lucene_LockManager::releaseReadLock($this->_directory);
+
+        		if (strpos($e->getMessage(), 'Can\'t obtain exclusive index lock') === false) {
+        			throw $e;
+        		} else {
+        			throw new Zend_Search_Lucene_Exception('Can\'t create index. It\'s under processing now');
+        		}
+        	}
+
+            if ($this->_generation == -1) {
+                // Directory doesn't contain existing index, start from 1
+                $this->_generation = 1;
+                $nameCounter = 0;
+            } else {
+                // Directory contains existing index
+                $segmentsFile = $this->_directory->getFileObject(self::getSegmentFileName($this->_generation));
+                $segmentsFile->seek(12); // 12 = 4 (int, file format marker) + 8 (long, index version)
+
+                $nameCounter = $segmentsFile->readInt();
+                $this->_generation++;
+            }
+
+            Zend_Search_Lucene_Index_Writer::createIndex($this->_directory, $this->_generation, $nameCounter);
+
+            Zend_Search_Lucene_LockManager::releaseWriteLock($this->_directory);
+        }
+
+        if ($this->_generation == -1) {
+            throw new Zend_Search_Lucene_Exception('Index doesn\'t exists in the specified directory.');
+        } else if ($this->_generation == 0) {
+            $this->_readPre21SegmentsFile();
+        } else {
+            $this->_readSegmentsFile();
         }
     }
 
@@ -276,8 +538,8 @@ class Zend_Search_Lucene implements Zend_Search_Lucene_Interface
 
         $this->commit();
 
-        // Free shared lock
-        $this->_lock->unlock();
+        // Release "under processing" flag
+        Zend_Search_Lucene_LockManager::releaseReadLock($this->_directory);
 
         if ($this->_closeDirOnExit) {
             $this->_directory->close();
@@ -327,13 +589,12 @@ class Zend_Search_Lucene implements Zend_Search_Lucene_Interface
     /**
      * Returns an instance of Zend_Search_Lucene_Index_Writer for the index
      *
-     * @internal
      * @return Zend_Search_Lucene_Index_Writer
      */
-    public function getIndexWriter()
+    private function _getIndexWriter()
     {
         if (!$this->_writer instanceof Zend_Search_Lucene_Index_Writer) {
-            $this->_writer = new Zend_Search_Lucene_Index_Writer($this->_directory, $this->_segmentInfos);
+            $this->_writer = new Zend_Search_Lucene_Index_Writer($this->_directory, $this->_segmentInfos, $this->_formatVersion);
         }
 
         return $this->_writer;
@@ -476,7 +737,7 @@ class Zend_Search_Lucene implements Zend_Search_Lucene_Interface
      */
     public function getMaxBufferedDocs()
     {
-        return $this->getIndexWriter()->maxBufferedDocs;
+        return $this->_getIndexWriter()->maxBufferedDocs;
     }
 
     /**
@@ -491,7 +752,7 @@ class Zend_Search_Lucene implements Zend_Search_Lucene_Interface
      */
     public function setMaxBufferedDocs($maxBufferedDocs)
     {
-        $this->getIndexWriter()->maxBufferedDocs = $maxBufferedDocs;
+        $this->_getIndexWriter()->maxBufferedDocs = $maxBufferedDocs;
     }
 
     /**
@@ -508,7 +769,7 @@ class Zend_Search_Lucene implements Zend_Search_Lucene_Interface
      */
     public function getMaxMergeDocs()
     {
-        return $this->getIndexWriter()->maxMergeDocs;
+        return $this->_getIndexWriter()->maxMergeDocs;
     }
 
     /**
@@ -525,7 +786,7 @@ class Zend_Search_Lucene implements Zend_Search_Lucene_Interface
      */
     public function setMaxMergeDocs($maxMergeDocs)
     {
-        $this->getIndexWriter()->maxMergeDocs = $maxMergeDocs;
+        $this->_getIndexWriter()->maxMergeDocs = $maxMergeDocs;
     }
 
     /**
@@ -547,7 +808,7 @@ class Zend_Search_Lucene implements Zend_Search_Lucene_Interface
      */
     public function getMergeFactor()
     {
-        return $this->getIndexWriter()->mergeFactor;
+        return $this->_getIndexWriter()->mergeFactor;
     }
 
     /**
@@ -569,7 +830,7 @@ class Zend_Search_Lucene implements Zend_Search_Lucene_Interface
      */
     public function setMergeFactor($mergeFactor)
     {
-        $this->getIndexWriter()->mergeFactor = $mergeFactor;
+        $this->_getIndexWriter()->mergeFactor = $mergeFactor;
     }
 
     /**
@@ -755,7 +1016,7 @@ class Zend_Search_Lucene implements Zend_Search_Lucene_Interface
         }
 
         $fdxFile = $segmentInfo->openCompoundFile('.fdx');
-        $fdxFile->seek( ($id-$segmentStartId)*8, SEEK_CUR );
+        $fdxFile->seek(($id-$segmentStartId)*8, SEEK_CUR);
         $fieldValuesPosition = $fdxFile->readLong();
 
         $fdtFile = $segmentInfo->openCompoundFile('.fdt');
@@ -1012,8 +1273,10 @@ class Zend_Search_Lucene implements Zend_Search_Lucene_Interface
      */
     public function addDocument(Zend_Search_Lucene_Document $document)
     {
-        $this->getIndexWriter()->addDocument($document);
+        $this->_getIndexWriter()->addDocument($document);
         $this->_docCount++;
+
+        $this->_hasChanges = true;
     }
 
 
@@ -1036,17 +1299,11 @@ class Zend_Search_Lucene implements Zend_Search_Lucene_Interface
     public function commit()
     {
         if ($this->_hasChanges) {
-            foreach ($this->_segmentInfos as $segInfo) {
-                $segInfo->writeChanges();
-            }
-
-            $this->_hasChanges = false;
-        }
-
-        if ($this->_writer !== null) {
-            $this->_writer->commit();
+            $this->_getIndexWriter()->commit();
 
             $this->_updateDocCount();
+
+            $this->_hasChanges = false;
         }
     }
 
@@ -1062,7 +1319,7 @@ class Zend_Search_Lucene implements Zend_Search_Lucene_Interface
         $this->commit();
 
         if (count($this->_segmentInfos) > 1 || $this->hasDeletions()) {
-            $this->getIndexWriter()->optimize();
+            $this->_getIndexWriter()->optimize();
             $this->_updateDocCount();
         }
     }
@@ -1096,9 +1353,7 @@ class Zend_Search_Lucene implements Zend_Search_Lucene_Interface
                 $result[] = $segmentInfo->currentTerm();
             }
 
-            $segmentInfo->nextTerm();
-            // check, if segment dictionary is finished
-            if ($segmentInfo->currentTerm() !== null) {
+            if ($segmentInfo->nextTerm() !== null) {
                 // Put segment back into the priority queue
                 $segmentInfoQueue->put($segmentInfo);
             }
@@ -1108,6 +1363,125 @@ class Zend_Search_Lucene implements Zend_Search_Lucene_Interface
     }
 
 
+    /**
+     * Terms stream queue
+     *
+     * @var Zend_Search_Lucene_Index_SegmentInfoPriorityQueue
+     */
+    private $_termsStreamQueue = null;
+
+    /**
+     * Last Term in a terms stream
+     *
+     * @var Zend_Search_Lucene_Index_Term
+     */
+    private $_lastTerm = null;
+
+    /**
+     * Reset terms stream.
+     */
+    public function resetTermsStream()
+    {
+        $this->_termsStreamQueue = new Zend_Search_Lucene_Index_SegmentInfoPriorityQueue();
+
+        foreach ($this->_segmentInfos as $segmentInfo) {
+            $segmentInfo->reset();
+
+            // Skip "empty" segments
+            if ($segmentInfo->currentTerm() !== null) {
+                $this->_termsStreamQueue->put($segmentInfo);
+            }
+        }
+
+        $this->nextTerm();
+    }
+
+    /**
+     * Skip terms stream up to specified term preffix.
+     *
+     * Prefix contains fully specified field info and portion of searched term
+     *
+     * @param Zend_Search_Lucene_Index_Term $prefix
+     */
+    public function skipTo(Zend_Search_Lucene_Index_Term $prefix)
+    {
+        $segments = array();
+
+        while (($segmentInfo = $this->_termsStreamQueue->pop()) !== null) {
+            $segments[] = $segmentInfo;
+        }
+
+        foreach ($segments as $segmentInfo) {
+            $segmentInfo->skipTo($prefix);
+
+            if ($segmentInfo->currentTerm() !== null) {
+                $this->_termsStreamQueue->put($segmentInfo);
+            }
+        }
+
+        $this->nextTerm();
+    }
+
+    /**
+     * Scans terms dictionary and returns next term
+     *
+     * @return Zend_Search_Lucene_Index_Term|null
+     */
+    public function nextTerm()
+    {
+        while (($segmentInfo = $this->_termsStreamQueue->pop()) !== null) {
+            if ($this->_termsStreamQueue->top() === null ||
+                $this->_termsStreamQueue->top()->currentTerm()->key() !=
+                            $segmentInfo->currentTerm()->key()) {
+                // We got new term
+                $this->_lastTerm = $segmentInfo->currentTerm();
+
+                if ($segmentInfo->nextTerm() !== null) {
+                    // Put segment back into the priority queue
+                    $this->_termsStreamQueue->put($segmentInfo);
+                }
+
+                return $this->_lastTerm;
+            }
+
+            if ($segmentInfo->nextTerm() !== null) {
+                // Put segment back into the priority queue
+                $this->_termsStreamQueue->put($segmentInfo);
+            }
+        }
+
+        // End of stream
+        $this->_lastTerm = null;
+
+        return null;
+    }
+
+    /**
+     * Returns term in current position
+     *
+     * @return Zend_Search_Lucene_Index_Term|null
+     */
+    public function currentTerm()
+    {
+        return $this->_lastTerm;
+    }
+
+    /**
+     * Close terms stream
+     *
+     * Should be used for resources clean up if stream is not read up to the end
+     */
+    public function closeTermsStream()
+    {
+        while (($segmentInfo = $this->_termsStreamQueue->pop()) !== null) {
+            $segmentInfo->closeTermsStream();
+        }
+
+        $this->_termsStreamQueue = null;
+        $this->_lastTerm         = null;
+    }
+
+
     /*************************************************************************
     @todo UNIMPLEMENTED
     *************************************************************************/
